import re
import sys


def truncate_comment(line):
    in_comment = False
    for i, c in enumerate(line):
        if c == '"':
            in_comment = not in_comment
        elif not in_comment and i < len(line) - 1 and line[i:i+2] == '//':
            return line[:i]
    return line


def preprocess(file, outfile=sys.stdout):
    regex = re.compile('^def|^while|^for|^if|^else')
    prev_tabs = 0
    out_line = None
    num_tabs = 0
    out_code = []
    
    compressed = []
    prev_line = "" 
    line = ""
    for line in file:
        left_str = line.strip(' \n \t \r')

        if (len(prev_line.strip(' \n \t \r')) > 0 or left_str.startswith('def ')):
            compressed.append(prev_line)
        prev_line = line
    if len(line.strip(' \n \t \r')) > 0:
        compressed.append(line)
    
    for line in compressed:
        leading_spaces = 4
        num_tabs = len(line) - len(line.lstrip('\t'))
        num_spaces = len(line) - len(line.lstrip(' '))
        if (num_tabs > 0 and num_spaces > 0):
            raise ValueError('Mixing spaces and tabs')
        else:
            num_tabs = max(num_tabs, num_spaces / leading_spaces)

        line = truncate_comment(line)
        line = line.strip(' \n \t \r')
        
        if (len(line) > 0):
            out_line = line.rstrip(' \n \t \r')
            if line.endswith(':') and re.match(regex, line):
                out_line = line[:len(line) - 1] + '~\n'
            elif len(line) > 0:
                out_line = out_line + ';\n'
            if num_tabs < prev_tabs:
                out_line = '$' * (prev_tabs - num_tabs) + out_line
                if not out_line.endswith('\n'):
                    out_line = out_line + '\n'
            out_code.append(out_line)
            prev_tabs = num_tabs
    end_dollars = num_tabs * '$'
    out_code.append(end_dollars)
    # print(''.join(out_code))
    outfile.write(''.join(out_code))


if __name__ == '__main__':
    prog_name = sys.argv[1]
    outfile = sys.stdout
    if len(sys.argv) > 2:
        outfile = open(sys.argv[2], 'w')
    
    with open(prog_name, 'r') as file:
        preprocess(file, outfile)

    if len(sys.argv) > 2:
        outfile.close()
