open Printf

type action = Sast | SastNoPreprocess | Compile | CompileNoPreprocess | CompileAndRun

let read_file filename =
  let lines = ref [] in
  let chan = open_in filename in
    try
      while true; do
        lines := input_line chan :: !lines
      done; !lines
    with End_of_file ->
      close_in chan;
      List.rev !lines

(* Usage: ./pseudo [myprogram.clrs myprogram.ll] [-s myprogram.clrs] [-sp myprogram.clrs] *)
let _ =
  (* Get action from flag *)
  let action =
    if Sys.argv.(1) = "-s"
    then Sast
    else (
      if Sys.argv.(1) = "-sp"
      then SastNoPreprocess
      else (
        if Sys.argv.(1) = "-p"
        then CompileNoPreprocess
        else (
          if Array.length Sys.argv > 2
          then Compile
          else CompileAndRun
        )
      )
    )
  in
  (* Grab correct source code file *)
  let infile_arg_index =
    if action = Sast || action = SastNoPreprocess || action = CompileNoPreprocess
    then 2
    else 1
  in
  let infile_name = Sys.argv.(infile_arg_index) in

  (* Pre-process if compile option is selected *)
  let processed_infile_name =
    if action = Sast || action = Compile || action = CompileAndRun
    then (* need to preprocess *)
      let command_str = sprintf ("python preprocessor.py %s %s") (infile_name) ("temp.pclrs") in
      let _ = Sys.command(command_str)
      in "temp.pclrs"
    else infile_name
  in
  (* Scan and Parse *)
  let infile_str = List.fold_left (fun s lst -> s ^ lst) "" (read_file processed_infile_name) in
  let _ = Sys.command(sprintf "rm -f %s" "temp.pclrs") in
  let lexbuf = Lexing.from_string infile_str in 
  let ast = Parser.program Scanner.token lexbuf in
  let sast, obj_table, obj_to_id_hash = Infer.infer_all ast in

  match action with
    | Compile | CompileNoPreprocess ->
      let m = Codegen.translate sast obj_table obj_to_id_hash in
      Llvm_analysis.assert_valid_module m;
      let outfile_index =
        if action = Compile
        then 2
        else 3
      in
      let outfile_name = Sys.argv.(outfile_index) in
      let oc = open_out outfile_name in
      Printf.fprintf oc "%s" (Llvm.string_of_llmodule m);
      close_out oc;
    | CompileAndRun -> let m = Codegen.translate sast obj_table obj_to_id_hash in
      Llvm_analysis.assert_valid_module m;
      let lli = "lli-3.7" in
      let run_str = sprintf "%s <<< '%s'" (lli) (Llvm.string_of_llmodule m) in
      let _ = Sys.command(run_str) in ()
    | Sast | SastNoPreprocess -> Printer.print_sast sast
