open Ast

type data_type =
  | Num
  | Bool
  | String
  | Void
  | List of data_type
  | Dict of data_type * data_type
  | Undetermined
  | T of string
  | Object of int (* object type id *)

type aexpr =
  | ANumLit of float * data_type
  | ABoolLit of bool * data_type
  | AStringLit of string * data_type
  | AVal of string * data_type
  | ABinop of aexpr * binop * aexpr * data_type
  | AUnop of unop * aexpr * data_type
  | AAssign of string * aexpr * data_type
  | ACall of string * aexpr list * data_type
  | ANoexpr of data_type
  (* List Ops *)
  | AListDecl of aexpr list * data_type
  | AListInsert of aexpr * aexpr * aexpr * data_type
  | AListPush of aexpr * aexpr * data_type
  | AListRemove of aexpr * aexpr * data_type
  | AListPop of aexpr * data_type
  | AListDequeue of aexpr * data_type
  | AListLength of aexpr * data_type
  | AListGet of aexpr * aexpr * data_type
  | AListSet of aexpr * aexpr * aexpr * data_type
  (* Dict Ops *)
  | ADictDecl of aexpr list * aexpr list * data_type
  | ADictMem of aexpr * aexpr * data_type
  | ADictDelete of aexpr * aexpr * data_type
  | ADictSize of aexpr * data_type
  | ADictFind of aexpr * aexpr * data_type
  | ADictMap of aexpr * aexpr * aexpr * data_type
  (* Object Ops *)
  | AObjectField of aexpr * string * data_type
  | AObjectAssign of aexpr * string * aexpr * data_type

(* Statements *)
type astmt =
    ABlock of astmt list
  | AExpr of aexpr
  | AReturn of aexpr
  | ABreak
  | AContinue
  | AWhile of aexpr * astmt
  | AForIn of aexpr * aexpr * astmt
  | AForRange of aexpr * aexpr * aexpr * astmt
  | AIf of aexpr * astmt * astmt
  | APrint of aexpr
  | AObjectInit of (string * data_type) list

(* Function Declarations *)
type afunc_decl = {
  afname: string;
  aformals: (data_type * string) list;  (* Parameters *)
  abody: astmt list;      (* Function Body *)
  return: data_type;
}

(* Program entry point *)
type aprogram = afunc_decl list
