(* Ocamllex scanner for MicroC *)

{ open Parser }

let numeric = ['0'-'9']

rule token = parse
  [' ' '\t' '\r' '\n'] { token lexbuf } (* Whitespace *)
| "/*"     { comment lexbuf }           (* Comments *)
| '('      { LPAREN }
| ')'      { RPAREN }
| '{'      { LBRACE }
| '}'      { RBRACE }
| '['      { LBRACKET }
| ']'      { RBRACKET }
| ','      { COMMA }
| '.'      { DOT }
| ':'      { COLON }
| '~'      { TILDE }
| '$'      { DOLLAR }
| ';'      { SEMI }
(* BINOPS *)
| '+'      { PLUS }
| '-'      { MINUS }
| '*'      { TIMES }
| '/'      { DIVIDE }
| '^'      { CONCAT }
| "::"     { COMBINE }
| '='      { ASSIGN }
| "=="     { EQ }
| "!="     { NEQ }
| '<'      { LT }
| "<="     { LEQ }
| ">"      { GT }
| ">="     { GEQ }
| "&&" | "and"    { AND }
| "||" | "or"     { OR }
| "!" | "not"      { NOT }

(* STRUCTURE KEYWORDS *)
| "def"    { DEF }
| "if"     { IF }
| "else"   { ELSE }
| "elsif"  { ELSIF }
| "for"    { FOR }
| "while"  { WHILE }
| "return" { RETURN }
| "print"  { PRINT }
| "to"     { TO }
| "in"     { IN }
| "by"		 { BY }
| "continue" { CONTINUE }
| "break"  { BREAK }

(* LIST KEYWORDS *)
| "get" { GET }
| "set" { SET }
| "length" { LENGTH }
| "insert" { INSERT }
| "remove" { REMOVE }
| "push" | "enqueue" | "append" { PUSH }
| "pop" { POP }
| "dequeue" { DEQUEUE }

(* MAP KEYWORDS *)
| "mem" { MEM }
| "find" { FIND }
| "map" { MAP } 
| "delete" { DEL }
| "size" { SIZE }

(* OBJECT KEYWORDS *)
| "init" { INIT }

(* DATA TYPES *)
| "true" | "false" as boollit { BOOL_LITERAL(bool_of_string boollit)}
| numeric* '.' numeric+ 
| numeric+ '.'numeric* as floatlit 
	{ NUM_LITERAL(float_of_string floatlit)}

| numeric+ as intlit { NUM_LITERAL(float_of_string intlit) }
| '"' (([^ '"'] | "\\\"")* as strlit) '"' { STRING_LITERAL(strlit) }
| ['a'-'z' 'A'-'Z']['a'-'z' 'A'-'Z' '0'-'9' '_']* as lxm { ID(lxm) }
| eof { EOF }
| _ as char { raise (Failure("illegal character " ^ Char.escaped char)) } 


and comment = parse
  "*/" { token lexbuf }
| _    { comment lexbuf }
