open Parser
open Ast

type num =
  | Num_int of int
  | Num_float of float 

let stringify = function
  (* Punctuation *)
  | LPAREN -> "LPAREN"      | RPAREN    -> "RPAREN"
  | LBRACE -> "LBRACE"      | RBRACE    -> "RBRACE"
  | LBRACKET -> "LBRACKET"  | RBRACKET  -> "RBRACKET"
  | COMMA -> "COMMA"        | TILDE     -> "TILDE"
  | DOLLAR -> "DOLLAR"      | SEMI      -> "SEMI" 
  | DOT -> "DOT"

  (* Arithmetic Operators *)
  | PLUS -> "PLUS"          | MINUS     -> "MINUS"
  | TIMES -> "TIMES"        | DIVIDE    -> "DIVIDE"

  (* List Operators *)
  | COMBINE -> "COMBINE"    | PUSH      -> "PUSH"
  | INSERT  -> "INSERT"     | DEQUEUE   -> "DEQUEUE"
  | POP     -> "POP"        | REMOVE    -> "REMOVE"
  | LENGTH  -> "LENGTH"

  (* Dict Operation *)
  | GET     -> "GET"        | SET       -> "SET"
  | MEM     -> "MEM"        | FIND      -> "FIND"
  | DEL     -> "DEL"        | MAP       -> "MAP"
  | SIZE      -> "SIZE"

  (* String Operators *)
  | CONCAT -> "CONCAT"

  (* Relational Operators *)
  | EQ -> "EQ"    | NEQ -> "NEQ"
  | LT -> "LT"    | GT -> "GT"
  | LEQ -> "LEQ"  | GEQ -> "GEQ"

  (* Logical Operators & Keywords *)
  | AND -> "AND"   | OR -> "OR"
  | NOT -> "NOT"

  (* Assignment Operator *)
  | ASSIGN -> "ASSIGN"

  (* Conditional Operators *)
  | IF -> "IF" | ELSE -> "ELSE"
  | ELSIF -> "ELSIF"

  (* Control Flow *)
  | WHILE -> "WHILE" | FOR -> "FOR"
  | BY -> "BY" | TO -> "TO"
  | IN -> "IN" | CONTINUE -> "CONTINUE"
  | BREAK -> "BREAK"

  (* Function Symbols & Keywords *)
  | DEF -> "DEF"
  | COLON -> "COLON"
  | RETURN -> "RETURN"

  (* UTILS *)
  | PRINT -> "PRINT" (* | ASSERT -> "ASSERT" *)

  (* End-of-File *)
  | EOF -> "EOF"

  (* Identifiers *)
  | ID(string) -> "ID"

  (* Literals *)
  | NUM_LITERAL(num) -> "NUM_LITERAL"
  | STRING_LITERAL(string) -> "STRING_LITERAL"
  | BOOL_LITERAL(bool) -> "BOOL_LITERAL"
  (*| VOID_LITERAL -> "VOID_LITERAL"*)
  (* | NONE -> "NONE"*)

  (* Objects *)
  | INIT -> "INIT"

let _ = 
  let lexbuf = Lexing.from_channel stdin in
  let rec print_tokens = function
    | EOF -> " "
    | token ->
      print_endline (stringify token);
      print_tokens (Scanner.token lexbuf) in
  print_tokens (Scanner.token lexbuf)
